// A compute shader that writes into a texture, showing how tightly compute shaders can work with the rest of OpenGL.

// As delivered, this works:
// g++ -o imagehack imagehack.cpp CSutilities.c ../common/Linux/MicroGlut.c ../common/*.c -I ../common/ -I ../common/Linux/ -DGL_GLEXT_PROTOTYPES -lm -lGL -lX11

#include <stdio.h>
#include <stdlib.h>
#include <sys/times.h>
#include <GL/gl.h>
#define MAIN
#include "GL_utilities.h"
#include "LittleOBJLoader.h"
#include "MicroGlut.h"
#include "CSutilities.h"

GLfloat vertices[] = {	-0.5f,-0.5f,0.0f,
			-0.5f,0.5f,0.0f,
			0.5f,-0.5f,0.0f,
			0.5f,0.5f,0.0f };

GLfloat texcoord[] = {	0.0f, 0.0f,
			0.0f, 1.0f,
			1.0f, 0.0f,
			1.0f, 1.0f};
GLuint indicies[] = {0, 1, 2, 1, 2, 3};

Model *quad;
GLuint quadprogram;

void display(void)
{
	printError("pre display");

	// clear the screen
	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH);
	glClear(GL_COLOR_BUFFER_BIT);

	DrawModel(quad, quadprogram, "inPosition", NULL, "inTexCoord");
	
	printError("display");
	
//	glutSwapBuffers();
	glFlush();
}

int main(int argc, char **argv)
{
// Let MicroGlut create a GL context
	glutInit (&argc, argv);
	glutInitContextVersion(4, 5);
	glutInitWindowSize(512, 512);
	glutCreateWindow("Image Hack");
	glutDisplayFunc(display);
//	glutRepeatingTimer(100);

    
	printf ("Vendor: %s\n", glGetString (GL_VENDOR));
	printf ("Renderer: %s\n", glGetString (GL_RENDERER));
	printf ("Version: %s\n", glGetString (GL_VERSION));
	printf ("GLSL: %s\n", glGetString(GL_SHADING_LANGUAGE_VERSION));

// Load and compile the compute shader
	GLuint p =loadComputeShader("imagehack.cs");

	GLuint tex;
	
	// Create texture
	
	glGenTextures(1, &tex);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA32F, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glGenerateMipmap(GL_TEXTURE_2D);

	glBindImageTexture(0, tex, 0, GL_FALSE, 0, GL_READ_WRITE, GL_RGBA32F);
	printError("bind");

	glUseProgram(p);
	
	glUniform1i(glGetUniformLocation(p, "tex"), 0);

	glDispatchCompute(32, 32, 1 );
	glMemoryBarrier( GL_ALL_BARRIER_BITS );

	printError("compute");

	// Display the image
	
	quadprogram = loadShaders("textured.vert", "textured.frag");
	glUseProgram(quadprogram);
	glUniform1i(glGetUniformLocation(quadprogram, "tex"), 0);
	
	quad = LoadDataToModel((vec3 *)vertices, NULL, (vec2 *)texcoord, NULL,  indicies,  4,  6);
	
	glutMainLoop();
}

